/**
   How can the key listener object interact with
   the important variables in our program?

   In particular, we would like the key listener to
   modify the model and then update the frambuffer.
*/

import renderer.gui.FrameBufferPanel;
import renderer.framebuffer.FrameBuffer;
import renderer.scene.*;
import renderer.models.Sphere;
import renderer.pipeline.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class InteractiveFrame_v0
{
   public static void main(String[] args)
   {
      FrameBuffer fb = new FrameBuffer(600, 600);
      FrameBufferPanel fbp = new FrameBufferPanel(fb);
      JFrame jf = new JFrame("Interactive Frame with FrameBuffer");
      jf.add(fbp);
      jf.pack();
      jf.setVisible(true);
      jf.addKeyListener(new KListener_v0());

      Scene scene = new Scene();
      Model model = new Sphere(1.0);
      scene.addModel(model);

      for (Vertex v : model.vertexList)
      {
         v.z -= 2.0;
      }

      Pipeline.render(scene, fb.vp);
      fbp.update();
   }
}


// A private class definition.
class KListener_v0 implements KeyListener
{
   @Override public void keyPressed (KeyEvent e){}
   @Override public void keyReleased(KeyEvent e){}
   @Override public void keyTyped   (KeyEvent e)
   {
      System.out.println(e);
   }
}

/*
Here is a picture showing some relevant objects in the JVM's heap. In this
example, the state of the program is divided between the Scene, FrameBufferPanel,
and JFrame objects. The JFrame object is the source of events. The KListener_v0
object is the event handler object.

      JFrame object
+----------------------+
|                      |       KeyListener
|  List<KeyListener>---|------>>+------+
|                      |        |      |           KListener_v0 object
|                      |        +------+        +-------------------------+
|  addKeyListener()    |        |  ----|------>>|                         |
+----------------------+        +------+        | keyPressed(KeyEvent e)  |
an object that receives         |      |        | keyReleased(KeyEvent e) |
KeyEvent objects from           +------+        | keyTyped(KeyEvent e)    |
the JVM                                         |                         |
                                                +-------------------------+
                                                 an object that implements
                                                 the KeyListener interface
     KeyEvent object
   +-----------------+
   |  information    |        Scene object            FrameBufferPanel object
   |  from the OS    |      +-----------------+       +-----------------+
   |  about the      |      |  information    |       | pixel data for  |
   |  keyboard event |      |  about our 3D   |       | our rendered 3D |
   +-----------------+      |  graphics scene |       | graphics scene  |
                            +-----------------+       +-----------------+
*/
