/**
   The listener object holds references to the
   important objects in our program. We use the
   listener's constructor as the way to pass this
   information to the listener object.
*/

import renderer.gui.FrameBufferPanel;
import renderer.framebuffer.FrameBuffer;
import renderer.scene.*;
import renderer.models.Sphere;
import renderer.pipeline.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class InteractiveFrame_v1
{
   public static void main(String[] args)
   {
      FrameBuffer fb = new FrameBuffer(600, 600);
      FrameBufferPanel fbp = new FrameBufferPanel(fb);
      JFrame jf = new JFrame("Interactive Frame with FrameBuffer");
      jf.add(fbp);
      jf.pack();
      jf.setVisible(true);

      Scene scene = new Scene();

      jf.addKeyListener(new KListener_v1(fbp, scene));

      Model model = new Sphere(1.0);
      scene.addModel(model);

      for (Vertex v : model.vertexList)
      {
         v.z -= 2.0;
      }

      Pipeline.render(scene, fb.vp);
      fbp.update();
   }
}


// A private class definition.
class KListener_v1 implements KeyListener
{
   public FrameBufferPanel fbp = null;
   public Scene scene = null;

   public KListener_v1(FrameBufferPanel fbp,
                       Scene scene)
   {
      this.fbp = fbp;
      this.scene = scene;
   }

   @Override public void keyPressed (KeyEvent e){}
   @Override public void keyReleased(KeyEvent e){}
   @Override public void keyTyped   (KeyEvent e)
   {
      System.out.println(e);
      for (Vertex v : scene.modelList.get(0).vertexList)
      {
         v.z -= 0.2;
      }
      fbp.getFrameBuffer().clearFB();
      Pipeline.render(scene, fbp.getFrameBuffer().vp);
      fbp.update();
   }
}

/*
Here is a picture showing some relevant objects in the JVM's heap.

      JFrame object
+----------------------+
|                      |       KeyListener         KListener_v1 object
|  List<KeyListener>---|------>>+------+        +-------------------------+
|                      |        |      |        | FrameBufferPanel fbp    |
|                      |        +------+        | Scene scene             |
|  addKeyListener()    |        |  ----|------>>|                         |
+----------------------+        +------+        | keyPressed(KeyEvent e)  |
an object that receives         |      |        | keyReleased(KeyEvent e) |
KeyEvent objects from           +------+        | keyTyped(KeyEvent e)    |
the JVM                                         |                         |
                                                +-------------------------+
                                                 an object that implements
                                                 the KeyListener interface
     KeyEvent object
   +-----------------+
   |  information    |        Scene object            FrameBufferPanel object
   |  from the OS    |      +-----------------+       +-----------------+
   |  about the      |      |  information    |       | pixel data for  |
   |  keyboard event |      |  about our 3D   |       | our rendered 3D |
   +-----------------+      |  graphics scene |       | graphics scene  |
                            +-----------------+       +-----------------+
*/
