/**
   Solve the race condition by running the InteractiveFrame_v12
   constructor in the Java GUI Event Dispatch Thread (EDT) instead
   of running it in the program's main() thread. This prevents the
   EDT from trying to handle an event before the constructor (running
   in the main() thread) has finished constructng the scene.

   Using javax.swing.SwingUtilities.invokeLater() will not work with
   versions 1, 2, 4, 6, and 8, which are the versions that create
   their event handler object using the main() method, because we
   cannot run the main() method on the event dispatch thread. But
   the next program shows a way that does seem to fix those other
   versions.
*/

import renderer.gui.FrameBufferPanel;
import renderer.framebuffer.FrameBuffer;
import renderer.scene.*;
import renderer.models.Sphere;
import renderer.pipeline.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

@SuppressWarnings("serial")
public class InteractiveFrame_v12 extends JFrame
{
   public InteractiveFrame_v12()
   {
      super("Interactive Frame with FrameBuffer");
      FrameBuffer fb = new FrameBuffer(600, 600);
      FrameBufferPanel fbp = new FrameBufferPanel(fb);
      this.add(fbp);
      this.pack();
      this.setVisible(true);

      Scene scene = new Scene(); // Put scene in the scope of the KeyListener.

      this.addKeyListener(
         new KeyListener(){ // An anonymous inner class constructor.
            // Implement the three methods of the KeyListener interface.
            @Override public void keyPressed (KeyEvent e){}
            @Override public void keyReleased(KeyEvent e){}
            @Override public void keyTyped   (KeyEvent e)
            {
               System.out.println(e);
               for (Vertex v : scene.modelList.get(0).vertexList)
               {
                  v.z -= 0.2;
               }
               fb.clearFB();
               Pipeline.render(scene, fb.vp);
               fbp.update();
            }
         }
      );

      // Attempt to force a race condition by delaying this thread.
      try{Thread.sleep(2000);}catch(InterruptedException e){} // milliseconds

      Model model = new Sphere(1.0);
      scene.addModel(model);

      for (Vertex v : model.vertexList)
      {
         v.z -= 2.0;
      }

      Pipeline.render(scene, fb.vp);
      fbp.update();
   }//constructor


   public static void main(String[] args)
   {
      // We should call the program's constructor in the Java
      // GUI Event Dispatch Thread (EDT), otherwise we get a race
      // condition between the constructor (running in the main()
      // thread) and the very first KeyEvent (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         new Runnable() // An anonymous inner class constructor.
         {
            @Override public void run() // Implement the Runnable interface.
            {
               // Call the constructor that builds the gui.
               new InteractiveFrame_v12();
            }
         }
      );
   }
}
