/**
   This seems to be a different way to solve the startup race condition.
   Here we instantiate the inner class key listener object at the location
   in the code that is the most advantageous, from the point of view of
   capturing important local variables, but we don't register the key
   listener object until we get to the last line of code, so no events
   will be processed before all of our relevant objects have been
   constructed.

   This solution does not act the same way as the previous solution.
   The previous solution would queue events while the constructor
   for the handler was being delayed. This version loses events
   that occur while the constructor for the handler is being delayed.
*/

import renderer.gui.FrameBufferPanel;
import renderer.framebuffer.FrameBuffer;
import renderer.scene.*;
import renderer.models.Sphere;
import renderer.pipeline.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class InteractiveFrame_v13
{
   public static void main(String[] args)
   {
      FrameBuffer fb = new FrameBuffer(600, 600);
      final FrameBufferPanel fbp = new FrameBufferPanel(fb);
      JFrame jf = new JFrame("Interactive Frame with FrameBuffer");
      jf.add(fbp);
      jf.pack();
      jf.setVisible(true);

      final Scene scene = new Scene(); // Put scene in the scope of the KeyListener.

      KeyListener kl = // instantiate, but don't register, a key listener object
         new KeyListener(){ // An anonymous inner class constructor.
            // Implement the three methods of the KeyListener interface.
            @Override public void keyPressed (KeyEvent e){}
            @Override public void keyReleased(KeyEvent e){}
            @Override public void keyTyped   (KeyEvent e)
            {
               System.out.println(e);
               for (Vertex v : scene.modelList.get(0).vertexList)
               {
                  v.z -= 0.2;
               }
               fb.clearFB();
               Pipeline.render(scene, fb.vp);
               fbp.update();
            }
         };

      // Delaying this thread will not cause a race condition.
      try{Thread.sleep(2000);}catch(InterruptedException e){} // milliseconds

      Model model = new Sphere(1.0);
      scene.addModel(model);

      for (Vertex v : model.vertexList)
      {
         v.z -= 2.0;
      }

      Pipeline.render(scene, fb.vp);
      fbp.update();
      jf.addKeyListener(kl); // now register the key listener object
   }
}
