/**
   This program attemps to stop Java's Event Dispatch Thread (EDT).
   It turns out that if you kill the EDT, the JVM seems to create
   a new EDT.
*/

import renderer.gui.FrameBufferPanel;
import renderer.framebuffer.FrameBuffer;
import renderer.scene.*;
import renderer.models.Sphere;
import renderer.pipeline.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

@SuppressWarnings("serial")
public class InteractiveFrame_v15 extends JFrame
{
   private int count = 0;  // mutable instance variable

   public InteractiveFrame_v15()
   {
      super("Interactive Frame with FrameBuffer");
      FrameBuffer fb = new FrameBuffer(600, 600);
      FrameBufferPanel fbp = new FrameBufferPanel(fb);
      this.add(fbp);
      this.pack();
      this.setVisible(true);
      //int count = 0; // Non final local variables may not be referenced by inner classes!
      Scene scene = new Scene(); // Put scene in the scope of the KeyListener.

      this.addKeyListener(
         new KeyListener(){ // An anonymous inner class constructor.
            // Implement the three methods of the KeyListener interface.
            @Override public void keyPressed (KeyEvent e){}
            @Override public void keyReleased(KeyEvent e){}
            @SuppressWarnings("deprecation")
            @Override public void keyTyped   (KeyEvent e)
            {
               System.out.println(e);
               for (Vertex v : scene.modelList.get(0).vertexList)
               {
                  v.z -= 0.2;
               }
               fb.clearFB();
               Pipeline.render(scene, fb.vp);
               fbp.update();

               System.out.printf("\ncount = %d\n", ++count); // mutate count
               System.out.println(Thread.currentThread());
               System.out.println("Current thread id = " + Thread.currentThread().getId() + ".\n");
               if (count >= 5)
               {
                  System.out.println("Stopping the EDT (id = " + Thread.currentThread().getId() + ").\n");
                  Thread.currentThread().stop(); // try to stop the EDT (stop() is deprecated)
               }
            }
         }
      );

      Model model = new Sphere(1.0);
      scene.addModel(model);

      for (Vertex v : model.vertexList)
      {
         v.z -= 2.0;
      }

      Pipeline.render(scene, fb.vp);
      fbp.update();
   }//constructor


   public static void main(String[] args)
   {
      // We should call the program's constructor in the Java
      // GUI Event Dispatch Thread (EDT), otherwise we get a race
      // condition between the constructor (running in the main()
      // thread) and the very first KeyEvent (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         new Runnable() // An anonymous inner class constructor.
         {
            @Override public void run() // Implement the Runnable interface.
            {
               // Call the constructor that builds the gui.
               new InteractiveFrame_v15();
            }
         }
      );
   }
}


/*
Here is a picture showing some relevant objects in the JVM's heap.

                                         InteractiveFrame_v15 constructor
                                         activation record of final local
                                         variables
                                         +---------------------------------+
                                         | final FrameBufferPanel fbp      |
                                         | final Scene scene               |
                                         | final InteractiveFrame_v15 this |                           |
                                         |                             |   |
                                         +-----------/\----------------|---+
                                                     /\                |
                                                     ||                |
InteractiveFrame_v15 object                          }}                |
(is a JFrame object)                                 ||                |
+----------------------+<<---------------------------------------------+
|                      |       KeyListener           ||
|  List<KeyListener>---|------>>+------+             || nesting link
|                      |        |      |             ||  (closure)
|  int count           |        +------+        +----||-------------------+
|  addKeyListener()    |        |  ----|------>>|                         |
+----------------------+        +------+        | keyPressed(KeyEvent e)  |
an object that receives         |      |        | keyReleased(KeyEvent e) |
KeyEvent objects from           +------+        | keyTyped(KeyEvent e)    |
the JVM                                         |                         |
                                                +-------------------------+
                                                 an object that implements
                                                 the KeyListener interface
     KeyEvent object
   +-----------------+
   |  information    |        Scene object            FrameBufferPanel object
   |  from the OS    |      +-----------------+       +-----------------+
   |  about the      |      |  information    |       | pixel data for  |
   |  keyboard event |      |  about our 3D   |       | our rendered 3D |
   +-----------------+      |  graphics scene |       | graphics scene  |
                            +-----------------+       +-----------------+
*/
