/**
   The event handler class is a static nested class
   of our program class. The event handler object has
   access to the program class's private static variables.
   (This is NOT a common way to write an event handler.)
*/

import renderer.gui.FrameBufferPanel;
import renderer.framebuffer.FrameBuffer;
import renderer.scene.*;
import renderer.models.Sphere;
import renderer.pipeline.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class InteractiveFrame_v4
{
   private static FrameBufferPanel fbp; // Compare with InteractiveFrame_v2.
   private static Scene scene;          // Declared private is an improvement.

   public static void main(String[] args)
   {
      FrameBuffer fb = new FrameBuffer(600, 600);
      fbp = new FrameBufferPanel(fb);
      JFrame jf = new JFrame("Interactive Frame with FrameBuffer");
      jf.add(fbp);
      jf.pack();
      jf.setVisible(true);
      jf.addKeyListener(new KListener_v4());

      scene = new Scene();
      Model model = new Sphere(1.0);
      scene.addModel(model);

      for (Vertex v : model.vertexList)
      {
         v.z -= 2.0;
      }

      Pipeline.render(scene, fb.vp);
      fbp.update();
   }//main()


   // A static nested class definition.
   static class KListener_v4 implements KeyListener
   {
      @Override public void keyPressed (KeyEvent e){}
      @Override public void keyReleased(KeyEvent e){}
      @Override public void keyTyped   (KeyEvent e)
      {
         System.out.println(e);
         for (Vertex v : scene.modelList.get(0).vertexList)
         {
            v.z -= 0.2;
         }
         fbp.getFrameBuffer().clearFB();
         Pipeline.render(scene, fbp.getFrameBuffer().vp);
         fbp.update();
      }
   }//KListener_v4
}

/*
Here is a picture showing some relevant objects in the JVM's heap.

                                               InteractiveFrame_v4 class
                                              +===========================+
                                             / static FrameBufferPanel fbp \
                                             \ static Scene scene          /
                                             /                             \
      JFrame object                          \ staic main()                /
+----------------------+                     /                             \
|                      |       KeyListener   \                             /
|  List<KeyListener>---|------>>+------+     /  static KListener_v4 object \
|                      |        |      |     \ +-------------------------+ /
|                      |        +------+     / |                         | \
|  addKeyListener()    |        |  ----|----->>| keyPressed(KeyEvent e)  | /
+----------------------+        +------+     / | keyReleased(KeyEvent e) | \
an object that receives         |      |     \ | keyTyped(KeyEvent e)    | /
KeyEvent objects from           +------+     / |                         | \
the JVM                                      \ +-------------------------+ /
                                             /  an object that implements  \
                                             \  the KeyListener interface  /
                                             /                             \
                                             \                             /
     KeyEvent object                          +===========================+
   +-----------------+
   |  information    |        Scene object            FrameBufferPanel object
   |  from the OS    |      +-----------------+       +-----------------+
   |  about the      |      |  information    |       | pixel data for  |
   |  keyboard event |      |  about our 3D   |       | our rendered 3D |
   +-----------------+      |  graphics scene |       | graphics scene  |
                            +-----------------+       +-----------------+
*/
