/**
   The event handler class is a nested class
   of our program class. The event handler
   object has access to the program object's
   private instance variables.
*/

import renderer.gui.FrameBufferPanel;
import renderer.framebuffer.FrameBuffer;
import renderer.scene.*;
import renderer.models.Sphere;
import renderer.pipeline.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class InteractiveFrame_v5
{
   private FrameBufferPanel fbp = null; // Compare with InteractiveFrame_v3.
   private Scene scene = null;

   public InteractiveFrame_v5() // main() turns into a constructor!
   {
      FrameBuffer fb = new FrameBuffer(600, 600);
      fbp = new FrameBufferPanel(fb);
      JFrame jf = new JFrame("Interactive Frame with FrameBuffer");
      jf.add(fbp);
      jf.pack();
      jf.setVisible(true);
      jf.addKeyListener(new KListener_v5());

      scene = new Scene();
      Model model = new Sphere(1.0);
      scene.addModel(model);

      for (Vertex v : model.vertexList)
      {
         v.z -= 2.0;
      }

      Pipeline.render(scene, fb.vp);
      fbp.update();
   }//constructor


   // A (non-static) nested class definition.
   class KListener_v5 implements KeyListener
   {
      @Override public void keyPressed (KeyEvent e){}
      @Override public void keyReleased(KeyEvent e){}
      @Override public void keyTyped   (KeyEvent e)
      {
         System.out.println(e);
         for (Vertex v : scene.modelList.get(0).vertexList)
         {
            v.z -= 0.2;
         }
         fbp.getFrameBuffer().clearFB();
         Pipeline.render(scene, fbp.getFrameBuffer().vp);
         fbp.update();
      }
   }//KListener_v5


   public static void main(String[] args)
   {
      new InteractiveFrame_v5(); // Weird!
   }
}

/*
Here is a picture showing some relevant objects in the JVM's heap.

                                                InteractiveFrame_v5 object
                                                +------------------------------+
                                                | private FrameBufferPanel fbp |
                                                | private Scene scene          |
                                                |                              |
      JFrame object                             |                              |
+----------------------+                        +----------------/\------------+
|                      |       KeyListener                       /\
|  List<KeyListener>---|------>>+------+                         || nesting link
|                      |        |      |         KListener_v5    ||  (closure)
|                      |        +------+        +----------------||-------+
|  addKeyListener()    |        |  ----|------>>|                         |
+----------------------+        +------+        | keyPressed(KeyEvent e)  |
an object that receives         |      |        | keyReleased(KeyEvent e) |
KeyEvent objects from           +------+        | keyTyped(KeyEvent e)    |
the JVM                                         |                         |
                                                +-------------------------+
                                                 an object that implements
                                                 the KeyListener interface
     KeyEvent object
   +-----------------+
   |  information    |        Scene object            FrameBufferPanel object
   |  from the OS    |      +-----------------+       +-----------------+
   |  about the      |      |  information    |       | pixel data for  |
   |  keyboard event |      |  about our 3D   |       | our rendered 3D |
   +-----------------+      |  graphics scene |       | graphics scene  |
                            +-----------------+       +-----------------+
*/
